###########################################################################
#
#   Copyright (c) 2001 Morten W. Petersen <morten@esol.no>
#	Copyright (c) 2001 Thingamy Ltd.
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, 
#    MA 02111-1307 USA
#
#
###########################################################################

import time
import fcntl
import string
from Globals import HTMLFile, data_dir
from Globals import Persistent
from Globals import default__class_init__
import OFS.SimpleItem

manage_add_thread_safe_counter_form = HTMLFile('thread_safe_counter_add', globals())

def manage_add_thread_safe_counter(self, id='get_unique_id', start_count_at=0,
				REQUEST=None, RESPONSE=None):
	"""Add an instance of the thread_safe_counter class."""

	zope_path = string.join(self.getPhysicalPath()[1:], '/')

	filename = string.replace(zope_path, '/', '_') + '_' + id

	self._setObject(id, thread_safe_counter(id, start_count_at, filename))

	if RESPONSE is not None:

		RESPONSE.redirect(self.absolute_url(relative=1))


class thread_safe_counter(
	Persistent,
	OFS.SimpleItem.SimpleItem
	):
	"""A thread safe counter that goes easy on the undos."""

	meta_type = 'ThreadSafeCounter'

	__ac_permissions__ = ()

	_properties = \
		(
			{
			'id':'counter', 'type':'int',
			'id':'counter_commit', 'type':'int'
			}
		)
 

        def __init__(self, id, start, filename):
		"""Initializes the instance and allocates a lock."""

		self.filename = filename

		file = open("%s/%s" % (data_dir, self.filename), 'w')
		file.write(str(start))


	def index_html(self, REQUEST, RESPONSE):
		"""Returns a unique identifier."""

		return self()


	def get_file(self, mode):
		"""Returns the file in the given mode."""

		return open("%s/%s" % (data_dir, self.filename), mode)


	def get_lock(self):
		"""Tries to get a lock on the counter file."""

		while 1:

			try:

				file = self.get_file('r')

				fcntl.flock(file.fileno(), fcntl.LOCK_EX | fcntl.LOCK_NB)
				break

			except IOError:

				# Should we be able to tweak this value?
				# What's the sensible default setting?

				time.sleep(0.01)


	def release_lock(self):
		"""Releases the lock we have on the file."""

		file = self.get_file('r')
		fcntl.flock(file.fileno(), fcntl.LOCK_UN)


	def get_value(self):
		"""Gets the current counter value."""

		value = self.get_file('r').read()

		return value


	def increase(self):
		"""Returns a unique identifier."""

		counter = str(int(self.get_value()) + 1)

		file = self.get_file('w')
		file.write(counter)

		return counter


        def __call__(self):
		"""Returns a unique identifier."""

		try:

			self.get_lock()
			counter = self.increase()

	                return counter

		finally:

			self.release_lock()
